from .utilities.strings import caseSplit


class Menu:
    """
    This class takes a list of functions to put in a menu

    items
    in the format [{"Quit python", exit}]

    execute
    if true, the menu automatically executes the selected function
    """
    def __init__(self, items, title=None, subtitle=None, execute=True):
        assert type(items) is list
        self.items = []
        self.title = title
        self.subtitle = subtitle
        self.execute = bool(execute)
        for item in items:
            if callable(item):  # Is the menu item a function/callable variable?
                try:
                    name = caseSplit(item.__name__)
                    self.items.append({name: item})
                except AttributeError:
                    # I will assume it is quit
                    self.items.append({"Quit": item})
            elif type(item) is dict:
                # Dictionary item must be in format
                # {"String", functionVariable}
                self.items.append(item)

    def show(self):
        print("=" * 100)
        if self.title:
            print("\n" + self.title)
        if self.subtitle:
            print("\n" + self.subtitle)

        for i in range(len(self.items)):
            functionName = list(self.items[i].keys())[0]
            print("{0} : {1}".format(i, functionName))

        # Select option
        option = -1
        try:
            option = int(input("\nChoose an option: "))
        except ValueError:
            print("Not a number :(")
            self.show()
        except NameError:
            print("Not a number :(")
            return self.show()

        # Validate
        if 0 <= option <= len(self.items) - 1:
            chosenFunctionName = list(self.items[option].keys())[0]
            chosenFunction = self.items[option][chosenFunctionName]
        else:
            print("Number out of bounds")
            return self.show()

        if self.execute:
            chosenFunction()
        return chosenFunction


if __name__ == '__main__':
    camelCaseSplit("helloWorld")



def camelCaseSplit(string):
    """
    :param string:
    :return: string
    """
    word = string[0].upper()
    i = 1
    while i < len(string):
        char = string[i]
        if (word[i - 1].islower() or word[i - 1] == " ") and char.isupper():
            word += " " + char
        else:
            word += char
        i += 1

    return word


def snakeCaseSplit(string):
    """
    :param string:
    :return: string
    """

    word = string[0].upper()
    i = 1
    while i < len(string):
        char = string[i]
        if string[i] == "_":
            word += " "
        elif word[i - 1] == " ":
            word += char.upper()
        else:
            word += char
        i += 1

    return word


def caseSplit(string):
    """
    :param string:
    :return: string
    """
    if "_" in string:
        return snakeCaseSplit(string)
    return camelCaseSplit(string)



import json
import os


class Yoda:
    """
    This class takes a path to a JSON file and loads its contents.
    If the JSON file doesn't exist, it will construct the JSON file, save it and still get the required contents

    For some reason I was running this in python 2.7, note that it does not work in 2.7
    """
    def __init__(self, path, keys):
        self.path = path
        self.keys = keys  # used for when the JSON doesn't exist and constructs a json with these values

    def contents(self):
        contents = self.open()
        if contents == {}:
            contents = self.__createJSON()
        return contents

    def __createJSON(self):
        titles = self.keys
        contents = {}
        print("For each of the keys below, paste the value")
        for title in titles:
            try:
                value = str(input("{}: ".format(title)))
            except NameError:
                value = ":("
                print("You are not running python 3!")
            contents[title] = value

        self.write(contents)
        return contents

    def open(self):
        path = self.getPath()
        try:
            jsonFile = open(path)
            file = json.load(jsonFile)
            jsonFile.close()
            return file
        except IOError:
            if ".json" not in path:  # try opening the json file with .json
                self.setPath(path + ".json")
                return self.open()
            else:
                return {}
        except ValueError:
            return {}

    def write(self, contents):
        assert type(contents) is dict
        path = self.getPath()
        jsonFile = open(path, "w")
        json.dump(contents, jsonFile, indent=4)
        jsonFile.close()

    def delete(self):
        if os.path.exists(self.getPath()):
            os.remove(self.getPath())

    def getPath(self):
        return self.path

    def setPath(self, path):
        self.path = path

